import functools
from datetime import datetime
from typing import Any, Dict, Union, Callable
import re


class ValidationError(Exception):
    """Custom exception for validation errors"""

    pass


class InputValidator:
    """Class containing various validation methods"""

    @staticmethod
    def validate_email_address(email: str) -> bool:
        """Validate email address format"""
        if not email:
            return True  # Allow empty emails
        pattern = r"^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$"
        return bool(re.match(pattern, email))

    @staticmethod
    def validate_date_format(date_str: str) -> bool:
        """Validate date format (YYYY-MM-DD or YYYY-MM-DDTHH:MM:SSZ)"""
        if not date_str:
            return True  # Allow empty dates

        # Try different date formats
        formats = [
            "%Y-%m-%d",
            "%Y-%m-%dT%H:%M:%SZ",
            "%Y-%m-%dT%H:%M:%S.%fZ",
            "%Y-%m-%dT%H:%M:%S",
        ]

        for fmt in formats:
            try:
                datetime.strptime(date_str, fmt)
                return True
            except ValueError:
                continue
        return False

    @staticmethod
    def validate_time_format(time_str: str) -> bool:
        """Validate time format (HH:MM:SS)"""
        if not time_str:
            return True  # Allow empty times

        pattern = r"^([01]?[0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]$"
        return bool(re.match(pattern, time_str))

    @staticmethod
    def validate_range_value(range_val: str) -> bool:
        """Validate range parameter values"""
        if not range_val:
            return True  # Allow empty range

        valid_ranges = [
            "previous_year",
            "previous_month",
            "previous_week",
            "previous_day",
            "last_365_days",
            "last_30_days",
            "last_7_days",
            "last_24_hours",
            "last_12_hours",
            "last_8_hours",
            "last_hour",
            "last_30_minutes",
        ]
        return range_val in valid_ranges

    @staticmethod
    def validate_read_value(read_val: Union[str, bool]) -> bool:
        """Validate read parameter values"""
        if read_val in ["all", True, False]:
            return True
        return False

    @staticmethod
    def validate_importance_value(importance: str) -> bool:
        """Validate importance parameter values"""
        if not importance:
            return True  # Allow empty importance

        valid_importance = ["low", "normal", "high"]
        return importance.lower() in valid_importance


def validate_inputs(**validation_rules):
    """
    Decorator for validating function inputs

    Usage:
    @validate_inputs(
        inbox={'type': list, 'required': True, 'min_length': 1},
        range={'type': str, 'validator': InputValidator.validate_range_value},
        start_date={'type': str, 'validator': InputValidator.validate_date_format},
        sender_address={'type': list, 'item_validator': InputValidator.validate_email_address}
    )
    def my_function(inbox, range, start_date, sender_address):
        pass
    """

    def decorator(func: Callable) -> Callable:
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            # Get function signature to map args to kwargs
            import inspect

            sig = inspect.signature(func)
            bound_args = sig.bind(*args, **kwargs)
            bound_args.apply_defaults()

            # Validate each parameter
            for param_name, rules in validation_rules.items():
                if param_name in bound_args.arguments:
                    value = bound_args.arguments[param_name]
                    _validate_parameter(param_name, value, rules)

            return func(*args, **kwargs)

        return wrapper

    return decorator


def _validate_parameter(param_name: str, value: Any, rules: Dict) -> None:
    """Validate a single parameter based on rules"""

    # Check if required
    if rules.get("required", False) and (value is None or value == [] or value == ""):
        raise ValidationError(f"Parameter '{param_name}' is required")

    # Skip further validation if value is empty and not required
    if not value and not rules.get("required", False):
        return

    # Check type
    expected_type = rules.get("type")
    if expected_type and not isinstance(value, expected_type):
        raise ValidationError(
            f"Parameter '{param_name}' must be of type {expected_type.__name__}, got {type(value).__name__}"
        )

    # Check list constraints
    if isinstance(value, list):
        min_length = rules.get("min_length")
        max_length = rules.get("max_length")

        if min_length is not None and len(value) < min_length:
            raise ValidationError(
                f"Parameter '{param_name}' must have at least {min_length} items"
            )

        if max_length is not None and len(value) > max_length:
            raise ValidationError(
                f"Parameter '{param_name}' must have at most {max_length} items"
            )

        # Validate each item in the list
        item_validator = rules.get("item_validator")
        if item_validator:
            for i, item in enumerate(value):
                if not item_validator(item):
                    raise ValidationError(
                        f"Item {i} in parameter '{param_name}' is invalid: {item}"
                    )

    # Check string constraints
    if isinstance(value, str):
        min_length = rules.get("min_length")
        max_length = rules.get("max_length")

        if min_length is not None and len(value) < min_length:
            raise ValidationError(
                f"Parameter '{param_name}' must be at least {min_length} characters long"
            )

        if max_length is not None and len(value) > max_length:
            raise ValidationError(
                f"Parameter '{param_name}' must be at most {max_length} characters long"
            )

    # Check allowed values
    allowed_values = rules.get("allowed_values")
    if allowed_values and value not in allowed_values:
        raise ValidationError(
            f"Parameter '{param_name}' must be one of {allowed_values}, got '{value}'"
        )

    # Custom validator
    validator = rules.get("validator")
    if validator and not validator(value):
        raise ValidationError(f"Parameter '{param_name}' failed validation: {value}")


# Predefined validation rules for common email parameters
EMAIL_VALIDATION_RULES = {
    "inbox": {
        "type": list,
        "required": True,
        "min_length": 1,
        "item_validator": InputValidator.validate_email_address,
    },
    "range": {"type": str, "validator": InputValidator.validate_range_value},
    "start_date": {"type": str, "validator": InputValidator.validate_date_format},
    "end_date": {"type": str, "validator": InputValidator.validate_date_format},
    "start_time": {"type": str, "validator": InputValidator.validate_time_format},
    "end_time": {"type": str, "validator": InputValidator.validate_time_format},
    "subject": {"type": list},
    "sender_name": {"type": list},
    "sender_address": {
        "type": list,
        "item_validator": InputValidator.validate_email_address,
    },
    "recipients": {
        "type": list,
        "item_validator": InputValidator.validate_email_address,
    },
    "cc": {"type": list, "item_validator": InputValidator.validate_email_address},
    "bcc": {"type": list, "item_validator": InputValidator.validate_email_address},
    "folder_path": {"type": str},
    "drafts": {"type": bool},
    "archived": {"type": bool},
    "deleted": {"type": bool},
    "sent": {"type": bool},
    "read": {"validator": InputValidator.validate_read_value},
    "attachments": {"type": bool},
    "storage": {
        "type": (str, type(None)),
        "allowed_values": [None, "emails", "emails_and_attachments"],
    },
    "importance": {"type": str, "validator": InputValidator.validate_importance_value},
    "email_body": {"type": str, "max_length": 10000},
    "to_recipients": {
        "type": list,
        "item_validator": InputValidator.validate_email_address,
    },
    "cc_recipients": {
        "type": list,
        "item_validator": InputValidator.validate_email_address,
    },
    "bcc_recipients": {
        "type": list,
        "item_validator": InputValidator.validate_email_address,
    },
}



from yougotmail.retrieve.retrieve_emails import RetrieveEmails
from yougotmail.ai._ai_prompts import AGENT_INSTRUCTIONS
from yougotmail.ai._ai_tools import AI_TOOLS, TOOLS
from typing import Any, Dict


class AI:
    def __init__(self, client_id, client_secret, tenant_id, open_ai_api_key=None):
        self.retrieve_emails = RetrieveEmails(client_id, client_secret, tenant_id)
        self.open_ai_api_key = open_ai_api_key
        self.client_id = client_id
        self.client_secret = client_secret
        self.tenant_id = tenant_id

    def _ensure_ai_handler(self):
        """Lazy import of AI Handler to avoid immediate OpenAI dependency"""
        if not self.open_ai_api_key:
            raise ValueError(
                "OpenAI API key is required for AI functionality. Please provide it when initializing the YouGotMail class."
            )

        try:
            from yougotmail.ai._ai_handler import AIHandler

            return AIHandler(open_ai_api_key=self.open_ai_api_key)
        except ImportError:
            raise ImportError(
                "OpenAI package is not installed. Install it with pip install 'yougotmail[openai]'"
            )

    def ai_structured_output_from_email_body(
        self, *, instructions: str, email_body: str, schema: Dict[str, Any]
    ):
        ai_handler = self._ensure_ai_handler()
        schema = {
            "type": "json_schema",
            "json_schema": {
                "name": "email_schema",
                "description": "A schema for answering questions about the email and its contents",
                "schema": {
                    "type": "object",
                    "properties": {
                        **schema,
                    },
                    "required": list(schema.keys()),
                    "additionalProperties": False,
                },
            },
        }

        content_for_ai = f"""
            Here is the email content: {email_body}
            """

        return ai_handler.structured_outputs(
            prompt=f"Extract the following information from the email using the following instructions: {instructions}. Provide the output in the following schema: {schema}",
            schema=schema,
            content=content_for_ai,
            model="gpt-4.1",
        )

    def ai_agent_with_tools(self, inbox, prompt):
        try:
            ai_handler = self._ensure_ai_handler()
            return ai_handler.function_calling(
                instructions=AGENT_INSTRUCTIONS,
                prompt=prompt,
                model="gpt-4.1",
                tools=TOOLS,
                ai_tools=AI_TOOLS(
                    self.client_id, self.client_secret, self.tenant_id, inbox
                ),
            )
        except Exception as e:
            print(f"Error in ai_agent_with_tools: {e}")
            return None



import requests
import base64
from yougotmail._utils._utils import Utils
from yougotmail.send.send import Send
import os
from urllib.parse import urlparse, unquote
import json


class Reply:
    def __init__(self, client_id, client_secret, tenant_id):
        self.utils = Utils()
        self.send = Send(client_id, client_secret, tenant_id)
        self.token = self.utils._generate_MS_graph_token(
            client_id, client_secret, tenant_id
        )

    # def _draft_reply(self, inbox, email_id, reply_body):
    #     try:
    #         # URL encode the message ID for the API call
    #         encoded_email_id = quote(email_id, safe="")
    #         url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{encoded_email_id}/createReply"
    #         print(f"📝 Creating reply at URL: {url}")
    #         headers = {
    #             "Authorization": f"Bearer {self.token}"
    #         }

    #         email = self._get_email_by_id(inbox, email_id)
    #         sender = self._find_email_sender(inbox, email_id)

    #         # Extract email content safely
    #         email_subject = email.get("subject", "No Subject")
    #         email_body_content = ""
    #         if email.get("body") and email.get("body").get("content"):
    #             email_body_content = email.get("body").get("content")

    #         email_split = f"""<div style="border-top: 1px solid #ccc; margin: 20px 0; padding: 10px 0;">
    #         <strong>From:</strong> &lt;{sender}&gt;<br>
    #         <strong>Date:</strong> {email.get("receivedDateTime")}<br>
    #         <strong>To:</strong> {self.INBOX}<br>
    #         <strong>Subject:</strong> {email_subject}<br>
    #         <div style="margin-top: 10px;">
    #         {email_body_content}
    #         </div>
    #         </div>"""

    #         # Generate AI reply
    #         is_reply_needed, ai_reply_content = self.ai_reply_draft(
    #             email_subject, email_body_content
    #         )
    #         if is_reply_needed == False:
    #             print("❌ No reply needed to the email")
    #             return None
    #         else:
    #             data = {
    #                 "message": {
    #                     "subject": "Re: " + email_subject,
    #                     "toRecipients": [{"emailAddress": {"address": sender}}],
    #                     "body": {
    #                         "contentType": "HTML",
    #                         "content": ai_reply_content
    #                         + email_split,
    #                     },
    #                 }
    #             }

    #             response = requests.post(url, headers=headers, json=data)
    #             if response.status_code == 201:  # createReply returns 201 on success
    #                 print("✅ Reply draft created successfully")
    #                 return response.json()
    #             else:
    #                 print(
    #                     f"❌ Error creating reply draft: Status {response.status_code}, Response: {response.text}"
    #                 )
    #                 return None

    #     except Exception as e:
    #         print(f"❌ Exception in _draft_reply: {e}")
    #         return None

    def reply_to_email(
        self, inbox="", email_id="", reply_body="", cc_recipients=[], bcc_recipients=[]
    ):
        if inbox == "":
            raise Exception("Inbox is required")

        email_id = email_id.split("_")[-1]

        reply_recipients = []
        reply_cc_recipients = []
        reply_bcc_recipients = []

        sender = self._find_email_sender(inbox, email_id)
        reply_recipients.append(sender)

        existing_recipients = self._find_email_recipients(inbox, email_id)
        existing_cc_recipients = self._find_email_cc_recipients(inbox, email_id)

        if existing_recipients is not None and len(existing_recipients) > 0:
            reply_recipients.extend(existing_recipients)

        if existing_cc_recipients is not None and len(existing_cc_recipients) > 0:
            reply_cc_recipients.extend(existing_cc_recipients)

        if cc_recipients is not None:
            reply_cc_recipients.extend(cc_recipients)

        if bcc_recipients is not None:
            reply_bcc_recipients.extend(bcc_recipients)

        reply_recipients_formatted = []
        reply_cc_recipients_formatted = []
        reply_bcc_recipients_formatted = []

        for reply_recipient in reply_recipients:
            reply_recipients_formatted.append(
                {"emailAddress": {"address": reply_recipient}}
            )
        for reply_cc_recipient in reply_cc_recipients:
            reply_cc_recipients_formatted.append(
                {"emailAddress": {"address": reply_cc_recipient}}
            )
        for reply_bcc_recipient in reply_bcc_recipients:
            reply_bcc_recipients_formatted.append(
                {"emailAddress": {"address": reply_bcc_recipient}}
            )

        data = {
            "message": {
                "toRecipients": reply_recipients_formatted,
                "ccRecipients": reply_cc_recipients_formatted,
                "bccRecipients": reply_bcc_recipients_formatted,
                "body": {"contentType": "HTML", "content": reply_body},
            }
        }

        url = (
            f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}/reply"
        )

        headers = {
            "Authorization": f"Bearer {self.token}",
            "Content-type": "application/json",
        }

        try:
            response = requests.post(url, headers=headers, json=data)

            if response.status_code == 202:
                return {
                    "status": "success",
                    "message": "Email reply sent successfully",
                    "recipients": reply_recipients_formatted,
                    "cc_recipients": reply_cc_recipients_formatted,
                    "bcc_recipients": reply_bcc_recipients_formatted,
                }
            else:
                error_message = "Failed to send email"
                try:
                    error_data = response.json()
                    if "error" in error_data:
                        error_message = error_data["error"].get(
                            "message", error_message
                        )
                except json.JSONDecodeError:
                    error_message = f"HTTP {response.status_code}: {response.reason}"

                return {
                    "status": "error",
                    "message": error_message,
                    "status_code": response.status_code,
                }
        except requests.exceptions.RequestException as e:
            return {"status": "error", "message": f"Request failed: {str(e)}"}

    def _find_email_sender(self, inbox, email_id):
        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}"
        headers = {"Authorization": f"Bearer {self.token}"}
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            return response.json()["from"]["emailAddress"]["address"]
        else:
            return None

    def _get_email_by_id(self, inbox, email_id):
        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}"
        headers = {"Authorization": f"Bearer {self.token}"}
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            return None

    def _find_email_recipients(self, inbox, email_id):
        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}"
        headers = {"Authorization": f"Bearer {self.token}"}
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            recipients = []
            for recipient in response.json()["toRecipients"]:
                if recipient["emailAddress"]["address"] != inbox:
                    recipients.append(recipient["emailAddress"]["address"])
            return recipients
        else:
            return None

    def _find_email_cc_recipients(self, inbox, email_id):
        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}"
        headers = {"Authorization": f"Bearer {self.token}"}
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            cc_recipients = []
            for cc_recipient in response.json()["ccRecipients"]:
                cc_recipients.append(cc_recipient["emailAddress"]["address"])
            return cc_recipients
        else:
            return None

    def _add_attachments_to_email(self, inbox, id, attachments):
        for attachment in attachments:
            if attachment.startswith("http"):
                file_content = self._get_file_from_url(attachment)
            else:
                file_content = self._get_file_from_local_file(attachment)

            attachment = {
                "@odata.type": "#microsoft.graph.fileAttachment",
                "name": file_content["filename"],
                "contentBytes": self._encode_file_into_base64(file_content["content"]),
            }
            url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{id}/attachments"

            headers = {
                "Authorization": f"Bearer {self.token}",
                "Content-type": "application/json",
            }

            response = requests.post(url, headers=headers, json=attachment)
            response.raise_for_status()
        return {
            "status": "success",
            "message": "Attachment added to email successfully",
            "id": response.json()["id"],
        }

    def _get_file_from_local_file(self, file_path):
        with open(file_path, "rb") as file:
            content = file.read()
            filename = os.path.basename(file_path)
            file_extension = os.path.splitext(filename)[1]

            return {
                "content": content,
                "filename": filename,
                "file_extension": file_extension,
                "size": len(content),
            }

    def _get_file_from_url(self, url):
        response = requests.get(url)
        response.raise_for_status()

        # Extract filename from URL
        parsed_url = urlparse(url)
        filename_from_url = os.path.basename(unquote(parsed_url.path))

        # Try to get filename from Content-Disposition header
        filename_from_header = None
        content_disposition = response.headers.get("Content-Disposition", "")
        if "filename=" in content_disposition:
            filename_from_header = content_disposition.split("filename=")[1].strip(
                "\"'"
            )

        # Determine final filename (prefer header over URL)
        filename = filename_from_header or filename_from_url or "downloaded_file"

        # Extract file extension
        file_extension = os.path.splitext(filename)[1] if filename else ""

        return {
            "content": response.content,
            "filename": filename,
            "file_extension": file_extension,
            "size": len(response.content),
        }

    def _encode_file_into_base64(self, file_content):
        return base64.b64encode(file_content).decode("utf-8")



import json
from textwrap import dedent


class AIHandler:
    def __init__(self, open_ai_api_key):
        try:
            from openai import OpenAI

            self.client = OpenAI(api_key=open_ai_api_key)
        except ImportError:
            raise ImportError(
                "OpenAI package is not installed. Install it with pip install 'yougotmail[openai]'"
            )

    def _content(self, content):
        if isinstance(content, dict):
            content = json.dumps(content)
        return content

    def text_generation(self, instructions="", prompt="", model="gpt-4.1"):
        try:
            response = self.client.responses.create(
                model=model, instructions=dedent(instructions), input=prompt
            )
            return response.output_text
        except Exception as e:
            print(f"An error occurred in text_generation: {str(e)}")

    def structured_outputs(self, prompt="", schema="", content="", model="gpt-4.1"):
        try:
            completion = self.client.beta.chat.completions.parse(
                model=model,
                temperature=0.0,
                messages=[
                    {"role": "system", "content": dedent(prompt)},
                    {"role": "user", "content": self._content(content)},
                ],
                response_format=schema,
            )
            return json.loads(completion.choices[0].message.content)
        except Exception as e:
            print(f"An error occurred in structured_outputs: {str(e)}")

    def reasoning(self, prompt="", reasoning_effort="medium", model="o4-mini"):
        try:
            response = self.client.responses.create(
                model=model,
                reasoning={"effort": f"{reasoning_effort}"},
                input=[{"role": "user", "content": prompt}],
            )

            return response.output_text
        except Exception as e:
            print(f"An error occurred: {e}")

    def process_image(self, prompt="", image_url="", model="gpt-4.1"):
        try:
            response = self.client.responses.create(
                model=model,
                input=[
                    {
                        "role": "user",
                        "content": [
                            {"type": "input_text", "text": prompt},
                            {
                                "type": "input_image",
                                "image_url": image_url,
                            },
                        ],
                    }
                ],
            )
            return response.output_text
        except Exception as e:
            print(f"An error occurred in process_image: {str(e)}")

    def function_calling(
        self, instructions="", prompt="", model="gpt-4.1", tools=[], ai_tools=""
    ):
        try:
            # Initialize conversation history
            conversation_history = [
                {"role": "developer", "content": dedent(instructions)},
                {"role": "user", "content": prompt},
            ]

            while True:
                response = self.client.responses.create(
                    model=model,
                    input=conversation_history,
                    tools=tools,
                    tool_choice="auto",
                )
                tool_call = response.output[0]

                if tool_call.type == "function_call":
                    args = json.loads(tool_call.arguments)
                    result = ai_tools._ai_function_router(tool_call.name, args)

                    conversation_history.append(tool_call)
                    conversation_history.append(
                        {
                            "type": "function_call_output",
                            "call_id": tool_call.call_id,
                            "output": str(result),
                        }
                    )
                else:
                    # Add assistant's response to conversation history
                    conversation_history.append(
                        {"role": "assistant", "content": response.output_text}
                    )

                    # Display assistant's response to user
                    print("\nAssistant:", response.output_text)

                    # Get user input for continuation
                    print("\nUser (type 'exit' to end conversation):", end=" ")
                    user_input = input()

                    # Check if user wants to exit
                    if user_input.lower() == "exit":
                        return response.output_text

                    # Add user's input to conversation history
                    conversation_history.append({"role": "user", "content": user_input})

        except Exception as e:
            print(f"An error occurred in function_calling: {e}")
            raise



from yougotmail.send.send import Send

TOOLS = [
    {
        "type": "function",
        "name": "send_email",
        "description": "Send an email to a given recipient based on instructions received from the user.",
        "strict": True,
        "parameters": {
            "type": "object",
            "properties": {
                "subject": {
                    "type": "string",
                    "description": "Subject of the email",
                },
                "email_body": {
                    "type": "string",
                    "description": "Body of the email",
                },
                "to_recipients": {
                    "type": "array",
                    "description": "Email address of the recipient",
                    "items": {
                        "type": "string",
                    },
                },
            },
            "required": ["subject", "email_body", "to_recipients"],
            "additionalProperties": False,
        },
    }
]


class AI_TOOLS:
    def __init__(self, client_id, client_secret, tenant_id, inbox):
        self.send = Send(client_id, client_secret, tenant_id)
        self.inbox = inbox

    def _ai_function_router(self, name, args):
        if name == "send_email":
            return self.send_email(
                args["subject"], args["email_body"], args["to_recipients"]
            )
        else:
            return "Function not found"

    def send_email(self, subject, email_body, to_recipients):
        response = self.send.send_email(
            inbox=self.inbox,
            subject=subject,
            email_body=email_body,
            to_recipients=to_recipients,
        )
        print(f"Email sent: {response}")
        return response



from yougotmail import YouGotMail
import os
from dotenv import load_dotenv
import json

load_dotenv()

ygm = YouGotMail(
    client_id=os.environ.get("MS_CLIENT_ID"),
    client_secret=os.environ.get("MS_CLIENT_SECRET"),
    tenant_id=os.environ.get("MS_TENANT_ID"),
)


def test_get_emails():
    try:
        emails = ygm.get_emails(
            inbox=[os.environ.get("INBOX_1")],
            # range="last_7_days",
            start_date="2024-06-01",  # can be date: YYYY-MM-DD 
            start_time="00:00:00", # time in 00:00:00, all time is UTC
            end_date="2025-06-03",  # can be date: YYYY-MM-DD 
            end_time="14:00:00", # time in 00:00:00, all time is UTC
            # subject=["Bristol", "7009"],
            # sender_name=[],
            # sender_address=[],
            # recipients=[],
            # cc=[],
            # bcc=[],
            folder_path="0. General/Admin/Banking",
            # drafts=False,
            # archived=False,
            # deleted=False,
            # sent=False,
            # read="all",
            # attachments=True,
            # storage=None,
        )
        with open("emails.json", "w") as f:
            json.dump(emails, f, indent=4)
    except Exception as e:
        print(f"Error: {e}")

def test_get_conversation():
    try:
        conversation = ygm.get_conversation(
            inbox=os.environ.get("INBOX_1"),
            conversation_id=os.environ.get("CONVERSATION_ID"),
            # range="last_7_days",
            # start_date="",
            # start_time="",
            # end_date="",
            # end_time="",
            # subject="",
            # sender_name="",
            # sender_address="",
            # read="all",
            # attachments=False,
        )
        with open("conversation.json", "w") as f:
            json.dump(conversation, f, indent=4)
    except Exception as e:
        print(f"Error: {e}")



from yougotmail.send.send import Send
from yougotmail.retrieve.retrieve_emails import RetrieveEmails


class Quick:
    def __init__(
        self,
        client_id,
        client_secret,
        tenant_id,
        mongo_url="",
        mongo_db_name="",
        aws_access_key_id="",
        aws_secret_access_key="",
        region_name="",
        bucket_name="",
        email_collection="",
        conversation_collection="",
        attachment_collection="",
    ):
        self.send = Send(client_id, client_secret, tenant_id)
        self.retrieve = RetrieveEmails(
            client_id,
            client_secret,
            tenant_id,
            mongo_url=mongo_url,
            mongo_db_name=mongo_db_name,
            email_collection=email_collection,
            conversation_collection=conversation_collection,
            attachment_collection=attachment_collection,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
            bucket_name=bucket_name,
        )

    # Quick action functions for simplified usage
    def get_last_hour_emails(self, inbox):
        """Returns all emails from the last hour"""
        try:
            emails = self.retrieve.get_emails(inbox=inbox, range="last_hour")
            print(f"Found {len(emails)} emails in the last hour")
            return emails
        except Exception as e:
            print(f"Error in last_hour_emails: {e}")
            return None

    def get_unread_emails(self, inbox):
        """Returns all unread emails from the last 24 hours"""
        try:
            emails = self.retrieve.get_emails(
                inbox=inbox, range="last_24_hours", read=False
            )
            return emails
        except Exception as e:
            print(f"Error in unread_emails: {e}")
            return None



from yougotmail import YouGotMail
import os
from dotenv import load_dotenv

load_dotenv()

ygm = YouGotMail(
    client_id=os.environ.get("MS_CLIENT_ID"),
    client_secret=os.environ.get("MS_CLIENT_SECRET"),
    tenant_id=os.environ.get("MS_TENANT_ID"),
)


# def test_create_microsoft_graph_webhook():
#     try:
#         create_webhook = ygm.create_microsoft_graph_webhook(
#             inbox=os.environ.get("INBOX_1"),
#             api_url=os.environ.get("WEBHOOK_URL"),
#             client_state=os.environ.get("CLIENT_STATE"),
#         )
#         print(create_webhook)
#     except Exception as e:
#         print(f"Error: {e}")


def test_get_active_subscriptions_for_inbox():
    try:
        active_subscriptions = ygm.get_active_subscriptions_for_inbox(
            inbox=os.environ.get("INBOX_1")
        )
        print(active_subscriptions)
    except Exception as e:
        print(f"Error: {e}")


def test_renew_subscriptions():
    try:
        renew_subscriptions = ygm.renew_subscriptions(inbox=os.environ.get("INBOX_1"))
        print(renew_subscriptions)
    except Exception as e:
        print(f"Error: {e}")



import requests
from yougotmail._utils._utils import Utils
from yougotmail.retrieve.retrieval_utils import RetrievalUtils
from yougotmail.retrieve.retrieve_conversations import RetrieveConversations


class MoveDelete:
    def __init__(self, client_id, client_secret, tenant_id):
        self.utils = Utils()
        self.retrieval_utils = RetrievalUtils(client_id, client_secret, tenant_id)
        self.retrieve_conversations = RetrieveConversations(
            client_id, client_secret, tenant_id
        )
        self.token = self.utils._generate_MS_graph_token(
            client_id, client_secret, tenant_id
        )

    def move_email_to_folder(self, inbox="", email_id="", folder_path=""):
        folder_id = self.retrieval_utils._resolve_folder_path(folder_path, inbox)
        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}/move"
        headers = {"Authorization": f"Bearer {self.token}"}
        data = {"destinationId": folder_id}
        response = requests.post(url, headers=headers, json=data)
        if response.status_code in [200, 201]:
            return {
                "status": "success", 
                "message": "Email moved to folder ✨",
                "email_id": email_id,
                "sender_name": response.json()["sender"]["emailAddress"]["name"],
                "subject": response.json()["subject"],
                "folder_path": folder_path
            }
        else:
            return {
                "status": "error",
                "message": "Failed to move email to folder ❌",
                "error": response.text
            }

    def delete_email(self, inbox="", email_id=""):
        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/messages/{email_id}"
        headers = {"Authorization": f"Bearer {self.token}"}
        response = requests.delete(url, headers=headers)
        if response.status_code in [200, 204]:
            return {
                "status": "success",
                "message": "Email deleted ✨",
                "email_id": email_id
            }
        else:
            return {
                "status": "error",
                "message": "Failed to delete email ❌",
                "error": response.text
            }

    def delete_conversation_by_id(self, inbox="", conversation_id=""):
        url = f"https://graph.microsoft.com/v1.0/users/{inbox}/conversations/{conversation_id}"
        headers = {"Authorization": f"Bearer {self.token}"}
        response = requests.delete(url, headers=headers)
        if response.status_code in [200, 204]:
            return {
                "status": "success",
                "message": "Conversation deleted ✨",
                "conversation_id": conversation_id
            }
        else:
            return {
                "status": "error",
                "message": "Failed to delete conversation ❌",
                "error": response.text
            }

    def delete_conversation(
        self,
        inbox="",
        conversation_id="",
        range="last_365_days",
        start_date="",
        start_time="",
        end_date="",
        end_time="",
        subject="",
        sender_name="",
        sender_address="",
        read="all",
        attachments=False,
        storage=None,
    ):
        conversation = self.retrieve_conversations.get_conversation(
            inbox=inbox,
            conversation_id=conversation_id,
            range=range,
            start_date=start_date,
            start_time=start_time,
            end_date=end_date,
            end_time=end_time,
            subject=subject,
            sender_name=sender_name,
            sender_address=sender_address,
            read=read,
            attachments=attachments,
            storage=storage,
        )
        return self.delete_conversation_by_id(inbox, conversation["id"])
